@@ -7,12 +7,12 @@ use Carp;
 #==================================================================
 # Author    : Djibril Ousmanou
 # Copyright : 2009
-# Update    : 01/04/2009 13:56:41
+# Update    : 07/04/2009 11:54:30
 # AIM       : Create line chart
 #==================================================================
 
 use vars qw($VERSION);
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 use base qw/Tk::Derived Tk::Canvas/;
 use Tk::Balloon;
@@ -93,7 +93,7 @@ sub Populate {
     -noaxis       => [ 'PASSIVE', 'Noaxis',       'NoAxis',       0 ],
     -zeroaxisonly => [ 'PASSIVE', 'Zeroaxisonly', 'ZeroAxisOnly', 0 ],
     -zeroaxis     => [ 'PASSIVE', 'Zeroaxis',     'ZeroAxis',     0 ],
-    -longticks => [ 'PASSIVE', 'Longticks', 'LongTicks', 0 ],
+    -longticks    => [ 'PASSIVE', 'Longticks',    'LongTicks',    0 ],
 
     -xtickheight => [
       'PASSIVE', 'Xtickheight', 'XTickHeight',
@@ -127,6 +127,15 @@ sub Populate {
       'PASSIVE',   'Colordata',
       'ColorData', $CompositeWidget->{RefInfoDummies}->{Legend}{Colors}
     ],
+
+    # splined
+    -smoothline => [ 'PASSIVE', 'Smoothline', 'SmoothLine', 0 ],
+
+    # points
+    -pointline  => [ 'PASSIVE', 'Pointline',  'PointLine',  0 ],
+    -markersize => [ 'PASSIVE', 'Markersize', 'MarkerSize', 6 ],
+    -markers => [ 'PASSIVE', 'Markers', 'Markers', [ 1 .. 8 ] ],
+
   );
 
   $CompositeWidget->Delegates( DEFAULT => $CompositeWidget, );
@@ -202,7 +211,20 @@ sub _Balloon {
           $OtherColor
             = $CompositeWidget->{RefInfoDummies}->{Balloon}{ColorData}->[1];
         }
-        $CompositeWidget->itemconfigure( $LineTag, -fill => $OtherColor, );
+
+        # if -fill enabled
+        if ( $CompositeWidget->itemcget( $LineTag, -fill ) ) {
+          $CompositeWidget->itemconfigure( $LineTag, -fill => $OtherColor, );
+        }
+
+        # if -outline enabled
+        eval {
+          if ( $CompositeWidget->itemcget( $LineTag, -outline ) )
+          {
+            $CompositeWidget->itemconfigure( $LineTag, -outline => $OtherColor,
+            );
+          }
+        };
       }
     );
 
@@ -210,8 +232,19 @@ sub _Balloon {
       $LegendTag,
       '<Leave>',
       sub {
-        $CompositeWidget->itemconfigure( $LineTag,
-          -fill => $CompositeWidget->{RefInfoDummies}{Line}{$LineTag}{color}, );
+        if ( $CompositeWidget->itemcget( $LineTag, -fill ) ) {
+          $CompositeWidget->itemconfigure( $LineTag,
+            -fill => $CompositeWidget->{RefInfoDummies}{Line}{$LineTag}{color},
+          );
+        }
+        eval {
+          if ( $CompositeWidget->itemcget( $LineTag, -outline ) )
+          {
+            $CompositeWidget->itemconfigure( $LineTag,
+              -outline =>
+                $CompositeWidget->{RefInfoDummies}{Line}{$LineTag}{color}, );
+          }
+        };
       }
     );
   }
@@ -226,7 +259,7 @@ sub set_legend {
   unless ( defined $RefLegend ) {
     $CompositeWidget->_error(
       "Can't set -data in set_legend method. "
-        . "May be you have forgot to set the value\n"
+        . "May be you forgot to set the value\n"
         . "Ex : set_legend( -data => ['legend1', 'legend2', ...] );",
       1
     );
@@ -780,7 +813,7 @@ sub _xtick {
   my ($CompositeWidget) = @_;
 
   my $xvaluecolor = $CompositeWidget->cget( -xvaluecolor );
-  my $longticks    = $CompositeWidget->cget( -longticks );
+  my $longticks   = $CompositeWidget->cget( -longticks );
 
   # x coordinates y ticks on bottom x axis
   my $Xtickx1 = $CompositeWidget->{RefInfoDummies}->{Axis}{CxMin};
@@ -866,7 +899,7 @@ sub _xtick {
 sub _ytick {
   my ($CompositeWidget) = @_;
 
-  my $longticks    = $CompositeWidget->cget( -longticks );
+  my $longticks = $CompositeWidget->cget( -longticks );
   $CompositeWidget->{RefInfoDummies}->{Axis}{Yaxis}{TickNumber}
     = $CompositeWidget->cget( -yticknumber );
 
@@ -944,7 +977,6 @@ sub _ytick {
     );
   }
 
-
   # Display the minimale value
   $CompositeWidget->createText(
     $CompositeWidget->{RefInfoDummies}->{Axis}{CxMin} - (
@@ -962,29 +994,29 @@ sub _ytick {
     ],
   );
 
-    # Long tick
-    unless ( defined $longticks and $longticks == 1 ) {
-      $CompositeWidget->createLine(
-        $CompositeWidget->{RefInfoDummies}->{Axis}{Cx0},
-        $CompositeWidget->{RefInfoDummies}->{Axis}{CyMin} - $Space,
-        $CompositeWidget->{RefInfoDummies}->{Axis}{Cx0}
-          - $CompositeWidget->{RefInfoDummies}->{Axis}{Yaxis}{TickWidth},
-        $CompositeWidget->{RefInfoDummies}->{Axis}{CyMin} - $Space,
-        -tags => [
-          $CompositeWidget->{RefInfoDummies}->{TAGS}{yTick},
-          $CompositeWidget->{RefInfoDummies}->{TAGS}{AllTick}
-        ],
-      );
-    }
-
+  # Long tick
+  unless ( defined $longticks and $longticks == 1 ) {
+    $CompositeWidget->createLine(
+      $CompositeWidget->{RefInfoDummies}->{Axis}{Cx0},
+      $CompositeWidget->{RefInfoDummies}->{Axis}{CyMin} - $Space,
+      $CompositeWidget->{RefInfoDummies}->{Axis}{Cx0}
+        - $CompositeWidget->{RefInfoDummies}->{Axis}{Yaxis}{TickWidth},
+      $CompositeWidget->{RefInfoDummies}->{Axis}{CyMin} - $Space,
+      -tags => [
+        $CompositeWidget->{RefInfoDummies}->{TAGS}{yTick},
+        $CompositeWidget->{RefInfoDummies}->{TAGS}{AllTick}
+      ],
+    );
+  }
 
   return;
 }
 
-sub _ViewData {
+sub _ViewDataLines {
   my ($CompositeWidget) = @_;
 
   my $legendmarkercolors = $CompositeWidget->cget( -colordata );
+  my $smoothline         = $CompositeWidget->cget( -smoothline );
 
   # number of value for x axis
   $CompositeWidget->{RefInfoDummies}->{Data}{xtickNumber}
@@ -1021,11 +1053,9 @@ sub _ViewData {
         = $CompositeWidget->{RefInfoDummies}->{Axis}{Cy0}
         - (
         $data * $CompositeWidget->{RefInfoDummies}->{Axis}{Yaxis}{HeightUnit} );
-
       push( @PointsData, ( $x, $y ) );
       $NumberData++;
     }
-
     my $LineColor = $legendmarkercolors->[$IndexColor];
     unless ( defined $LineColor ) {
       $IndexColor = 0;
@@ -1035,9 +1065,10 @@ sub _ViewData {
 
     $CompositeWidget->createLine(
       @PointsData,
-      -fill  => $LineColor,
-      -tags  => [ $tag, $CompositeWidget->{RefInfoDummies}->{TAGS}{AllData} ],
-      -width => $CompositeWidget->cget( -linewidth ),
+      -fill   => $LineColor,
+      -tags   => [ $tag, $CompositeWidget->{RefInfoDummies}->{TAGS}{AllData} ],
+      -width  => $CompositeWidget->cget( -linewidth ),
+      -smooth => $smoothline,
     );
 
     $CompositeWidget->{RefInfoDummies}{Line}{$tag}{color} = $LineColor;
@@ -1049,6 +1080,94 @@ sub _ViewData {
   return 1;
 }
 
+sub _ViewDataPoints {
+  my ($CompositeWidget) = @_;
+
+  my $legendmarkercolors = $CompositeWidget->cget( -colordata );
+  my $markersize         = $CompositeWidget->cget( -markersize );
+  my $markers            = $CompositeWidget->cget( -markers );
+
+  # number of value for x axis
+  $CompositeWidget->{RefInfoDummies}->{Data}{xtickNumber}
+    = $CompositeWidget->{RefInfoDummies}->{Data}{NumberXValues};
+
+  # Space between x ticks
+  $CompositeWidget->{RefInfoDummies}->{Axis}{Xaxis}{SpaceBetweenTick}
+    = $CompositeWidget->{RefInfoDummies}->{Axis}{Xaxis}{Width}
+    / ( $CompositeWidget->{RefInfoDummies}->{Data}{xtickNumber} + 1 );
+
+  my $IdData      = 0;
+  my $IndexColor  = 0;
+  my $IndexMarker = 0;
+  foreach my $RefArrayData (
+    @{ $CompositeWidget->{RefInfoDummies}->{Data}{RefAllData} } )
+  {
+    if ( $IdData == 0 ) {
+      $IdData++;
+      next;
+    }
+
+    my $LineColor = $legendmarkercolors->[$IndexColor];
+    my $NumMarker = $markers->[$IndexMarker];
+    unless ( defined $LineColor ) {
+      $IndexColor = 0;
+      $LineColor  = $legendmarkercolors->[$IndexColor];
+    }
+    unless ( defined $NumMarker ) {
+      $IndexMarker = 0;
+      $NumMarker   = $markers->[$IndexMarker];
+    }
+    my $tag = $IdData . $CompositeWidget->{RefInfoDummies}->{TAGS}{Line};
+    $CompositeWidget->{RefInfoDummies}{Line}{$tag}{color} = $LineColor;
+
+    my $NumberData = 1;    # Number of data
+    my @PointsData;        # coordinate x and y
+    foreach my $data ( @{$RefArrayData} ) {
+      unless ( defined $data ) {
+        $NumberData++;
+        next;
+      }
+
+      # coordinates x and y values
+      my $x
+        = $CompositeWidget->{RefInfoDummies}->{Axis}{Cx0}
+        + $NumberData
+        * $CompositeWidget->{RefInfoDummies}->{Axis}{Xaxis}{SpaceBetweenTick};
+      my $y
+        = $CompositeWidget->{RefInfoDummies}->{Axis}{Cy0}
+        - (
+        $data * $CompositeWidget->{RefInfoDummies}->{Axis}{Yaxis}{HeightUnit} );
+
+      my %Option = (
+        -tags => [ $tag, $CompositeWidget->{RefInfoDummies}->{TAGS}{AllData} ],
+        -width => $CompositeWidget->cget( -linewidth ),
+      );
+      my $RefType = $CompositeWidget->_GetMarkerType($NumMarker);
+      if ( $RefType->[1] == 1 ) {
+        $Option{-fill} = $LineColor;
+      }
+      if ( $NumMarker =~ m{^[125678]$} ) {
+        $Option{-outline} = $LineColor;
+      }
+
+      $CompositeWidget->_CreateType(
+        x      => $x,
+        y      => $y,
+        pixel  => $markersize,
+        type   => $RefType->[0],
+        option => \%Option,
+      );
+      $NumberData++;
+    }
+
+    $IdData++;
+    $IndexColor++;
+    $IndexMarker++;
+  }
+
+  return 1;
+}
+
 sub plot {
   my ( $CompositeWidget, $RefData, %option ) = @_;
 
@@ -1062,11 +1181,12 @@ sub plot {
   }
 
   unless ( defined $RefData ) {
+    $CompositeWidget->_error("data not defined");
     return;
   }
 
   unless ( scalar @{$RefData} > 1 ) {
-    $CompositeWidget->_error("Warning : You must have at least 2 arrays");
+    $CompositeWidget->_error("You must have at least 2 arrays");
     return;
   }
 
@@ -1087,9 +1207,7 @@ sub plot {
       == $CompositeWidget->{RefInfoDummies}->{Data}{NumberXValues} )
     {
       $CompositeWidget->_error(
-        "Warning : Make sure that every array has the same size in plot data method",
-        1
-      );
+        "Make sure that every array has the same size in plot data method", 1 );
       return;
     }
 
@@ -1098,7 +1216,7 @@ sub plot {
 
       # substitute none real value
       foreach my $data ( @{$RefArray} ) {
-        unless ( defined $data and _isANumber($data) ) {
+        if ( defined $data and !_isANumber($data) ) {
           $data = $CompositeWidget->{RefInfoDummies}->{Data}{SubstitutionValue};
         }
       }
@@ -1132,6 +1250,8 @@ sub plot {
       = int( $CompositeWidget->{RefInfoDummies}->{Data}{MaxYValue} + 1 );
   }
 
+  $CompositeWidget->_GraphForDummiesConstruction;
+
   return 1;
 }
 
@@ -1155,7 +1275,14 @@ sub _GraphForDummiesConstruction {
   $CompositeWidget->_YLabelPosition();
   $CompositeWidget->_XLabelPosition();
   $CompositeWidget->_title();
-  $CompositeWidget->_ViewData();
+  if ( $CompositeWidget->cget( -pointline ) == 1 ) {
+    $CompositeWidget->_ViewDataPoints();
+  }
+  else {
+    $CompositeWidget->_ViewDataLines();
+  }
+
+  #
   unless ( $CompositeWidget->cget( -noaxis ) == 1 ) {
     $CompositeWidget->_xtick();
     $CompositeWidget->_ytick();
@@ -1424,7 +1551,7 @@ __END__
 
 =head1 NAME
 
-Tk::ForDummies::Graph::Lines - Extension of Canvas widget to create a line chart like GDGraph. 
+Tk::ForDummies::Graph::Lines - Extension of Canvas widget to create a line chart. 
 
 =head1 SYNOPSIS
 
@@ -1478,7 +1605,7 @@ The axes can be automatically scaled or set by the code.
 With this module it is possible to plot quantitative variables according to qualitative variables.
 
 When the mouse cursor passes over a plotted line or its entry in the legend, 
-the line and its entry will be turn to a color (that you can change) to help identify it. 
+the line and its entry will be turned to a color (that you can change) to help identify it. 
 
 You can use 3 methods to zoom (vertically, horizontally or both).
 
@@ -1495,7 +1622,9 @@ B<-yscrollincrement>
 =head1 WIDGET-SPECIFIC OPTIONS 
 
 Many options allow you to configure your chart as you want. 
-The default configuration have already OK, but you can change it.
+The default configuration is already OK, but you can change it.
+
+=head3 Options for all lines graph
 
 =over 4
 
@@ -1644,7 +1773,7 @@ Default : B<30>
 
 =item Switch:	B<-xvaluesregex>
 
-View the x values which will match with regex. It allow you to display tick on x axis and values 
+View the x values which will match with regex. It allows you to display tick on x axis and values 
 that you want. You can combine it with -xlabelskip to perform what you want to display if you have many dataset.
 
  
@@ -1935,11 +2064,89 @@ Default :
     'black',   '#FFCCFF', '#99CCFF', '#FF00CC', '#FF8000', '#006090',
   ],
 
-The default array contain 24 colors. If you have more than 24 samples, the next line 
+The default array contains 24 colors. If you have more than 24 samples, the next line 
 will have the color of the first array case (red).
 
 =back
 
+=head3 Options for spline lines graph
+
+=over 4
+
+=item Name:	B<Smoothline>
+
+=item Class:	B<SmoothLine>
+
+=item Switch:	B<-smoothline>
+
+To create lines chart as B<Bézier curve>.
+ 
+ -smoothline => 1, # 0 or 1
+
+Default : B<0>
+
+=back
+
+
+=head3 Options for point lines graph
+
+Theses option is specific to point lines chart creation.
+
+=over 4
+
+=item Name:	B<Pointline>
+
+=item Class:	B<PointLine>
+
+=item Switch:	B<-pointline>
+
+Sets to a true value to create point lines chart.  
+
+ -pointline => 1, # 0 or 1
+
+Default : B<0>
+
+=item Name:	B<Markersize>
+
+=item Class:	B<MarkerSize>
+
+=item Switch:	B<-markersize>
+
+The size of the markers used in points graphs, in pixels. 
+
+ -markersize => 10, # integer
+
+Default : B<6>
+
+=item Name:	B<Markers>
+
+=item Class:	B<Markers>
+
+=item Switch:	B<-markers>
+
+This controls the order of markers in points graphs. 
+This should be a reference to an array of numbers:
+
+ -markers => [3, 5, 6],
+
+  Available markers are: 
+  
+  1:  filled square 
+  2:  open square 
+  3:  horizontal cross
+  4:  diagonal cross
+  5:  filled diamond
+  6:  open diamond
+  7:  filled circle
+  8:  open circle
+  9:  horizontal line
+  10: vertical line
+
+Default : B<[1,2,3,4,5,6,7,8]>
+Note that the last two are not part of the default list.
+
+=back
+
 =head1 WIDGET METHODS
 
 The Canvas method creates a widget object. This object supports the 
@@ -1971,7 +2178,7 @@ will complain and refuse to compile the graph.
  my @NewData = (1,10,12,5,4);
  $GraphDummies>->(\@NewData);
 
-If your last chart have a legend, you have to add a legend entry for the new dataset. Otherwise, 
+If your last chart has a legend, you have to add a legend entry for the new dataset. Otherwise, 
 the legend chart will not be display (see below).
 
 =item *
@@ -2046,7 +2253,7 @@ and the point will be skipped.
 
 -substitutionvalue => I<real number>,
 
-If you have a missing value in a dataset, it will be replaced by a constant value.
+If you have a no real number value in a dataset, it will be replaced by a constant value.
 
 Default : B<0>
 
@@ -2074,7 +2281,7 @@ Redraw the chart.
 If you have used clearchart for any reason, it is possible to redraw the chart.
 Tk::ForDummies::Graph::Lines supports the configure and cget methods described in the L<Tk::options> manpage.
 If you use configure method to change a widget specific option, the modification will not be display. 
-if the chart was already displayed and if you not resize the widget, call B<redraw> method to 
+If the chart was already displayed and if you not resize the widget, call B<redraw> method to 
 resolv the bug.
 
  ...
@@ -2123,8 +2330,8 @@ Default : B<snow>
 
 -colordatamouse => I<Array reference>
 
-Specifie a array reference wich contains 2 colors. The first color specifies 
-the color of the line when mouse cursor passes over a entry in the legend. If the line 
+Specify an array reference wich contains 2 colors. The first color specifies 
+the color of the line when mouse cursor passes over an entry in the legend. If the line 
 has the same color, the second color will be used.
 
  -colordatamouse => ["blue", "green"],
@@ -2277,7 +2484,7 @@ zoom the chart the y axis.
 
 =head1 AUTHOR
 
-Djibril Ousmanou, C<< <djibrilo at yahoo.fr> >>
+Djibril Ousmanou, C<< <djibel at cpan.org> >>
 
 =head1 BUGS
 
@@ -2456,7 +2663,7 @@ Create a zoom Menu with the chart.
 
 See L<Tk::Canvas> for details of the standard options.
 
-See L<Tk::ForDummies::Graph>, L<GD::Graph>, L<Tk::Graph>, L<Tk::LineGraph>, L<Tk::PlotDataset>
+See L<Tk::ForDummies::Graph>, L<Tk::ForDummies::Graph::FAQ>, L<GD::Graph>, L<Tk::Graph>, L<Tk::LineGraph>, L<Tk::PlotDataset>
 
 =head1 SUPPORT
 
